@@ -3,23 +3,20 @@ Scoring server for python model format.
 The passed int model is expected to have function:
    predict(pandas.Dataframe) -> pandas.DataFrame
 
-Input, expected intext/csv or application/json format,
+Input, expected in text/csv or application/json format,
 is parsed into pandas.DataFrame and passed to the model.
 
 Defines two endpoints:
     /ping used for health check
     /invocations used for scoring
 """
-from __future__ import print_function
-
 from collections import OrderedDict
 import flask
 import json
-from json import JSONEncoder
 import logging
 import numpy as np
+import os
 import pandas as pd
-from six import reraise
 import sys
 import traceback
 
@@ -29,9 +26,17 @@ import traceback
 # dependencies to the minimum here.
 # ALl of the mlfow dependencies below need to be backwards compatible.
 from mlflow.exceptions import MlflowException
+from mlflow.types import Schema
+from mlflow.utils import reraise
+from mlflow.utils.proto_json_utils import (
+    NumpyEncoder,
+    _dataframe_from_json,
+    _get_jsonable_obj,
+    parse_tf_serving_input,
+)
 
 try:
-    from mlflow.pyfunc import load_model
+    from mlflow.pyfunc import load_model, PyFuncModel
 except ImportError:
     from mlflow.pyfunc import load_pyfunc as load_model
 from mlflow.protos.databricks_pb2 import MALFORMED_REQUEST, BAD_REQUEST
@@ -53,32 +58,79 @@ CONTENT_TYPE_JSON_SPLIT_NUMPY = "application/json-numpy-split"
 CONTENT_TYPES = [
     CONTENT_TYPE_CSV,
     CONTENT_TYPE_JSON,
-    CONTENT_TYPE_JSON_RECORDS_ORIENTED,
-    CONTENT_TYPE_JSON_SPLIT_ORIENTED,
-    CONTENT_TYPE_JSON_SPLIT_NUMPY
+    CONTENT_TYPE_JSON_SPLIT_NUMPY,
 ]
 
+CONTENT_TYPE_FORMAT_RECORDS_ORIENTED = "pandas-records"
+CONTENT_TYPE_FORMAT_SPLIT_ORIENTED = "pandas-split"
+
+FORMATS = [CONTENT_TYPE_FORMAT_RECORDS_ORIENTED, CONTENT_TYPE_FORMAT_SPLIT_ORIENTED]
+
+PREDICTIONS_WRAPPER_ATTR_NAME_ENV_KEY = "PREDICTIONS_WRAPPER_ATTR_NAME"
+
 _logger = logging.getLogger(__name__)
 
 
-def parse_json_input(json_input, orient="split"):
+def infer_and_parse_json_input(json_input, schema: Schema = None):
+    """
+    :param json_input: A JSON-formatted string representation of TF serving input or a Pandas
+                       DataFrame, or a stream containing such a string representation.
+    :param schema: Optional schema specification to be used during parsing.
+    """
+    try:
+        decoded_input = json.loads(json_input)
+    except json.decoder.JSONDecodeError:
+        _handle_serving_error(
+            error_message=(
+                "Failed to parse input from JSON. Ensure that input is a valid JSON"
+                " formatted string."
+            ),
+            error_code=MALFORMED_REQUEST,
+        )
+
+    if isinstance(decoded_input, list):
+        return parse_json_input(json_input=json_input, orient="records", schema=schema)
+    elif isinstance(decoded_input, dict):
+        if "instances" in decoded_input or "inputs" in decoded_input:
+            try:
+                return parse_tf_serving_input(decoded_input, schema=schema)
+            except MlflowException as ex:
+                _handle_serving_error(
+                    error_message=(ex.message), error_code=MALFORMED_REQUEST,
+                )
+        else:
+            return parse_json_input(json_input=json_input, orient="split", schema=schema)
+    else:
+        _handle_serving_error(
+            error_message=(
+                "Failed to parse input from JSON. Ensure that input is a valid JSON"
+                " list or dictionary."
+            ),
+            error_code=MALFORMED_REQUEST,
+        )
+
+
+def parse_json_input(json_input, orient="split", schema: Schema = None):
     """
     :param json_input: A JSON-formatted string representation of a Pandas DataFrame, or a stream
                        containing such a string representation.
     :param orient: The Pandas DataFrame orientation of the JSON input. This is either 'split'
                    or 'records'.
+    :param schema: Optional schema specification to be used during parsing.
     """
-    # pylint: disable=broad-except
+
     try:
-        return pd.read_json(json_input, orient=orient, dtype=False)
+        return _dataframe_from_json(json_input, pandas_orient=orient, schema=schema)
     except Exception:
         _handle_serving_error(
             error_message=(
                 "Failed to parse input as a Pandas DataFrame. Ensure that the input is"
                 " a valid JSON-formatted Pandas DataFrame with the `{orient}` orient"
                 " produced using the `pandas.DataFrame.to_json(..., orient='{orient}')`"
-                " method.".format(orient=orient)),
-            error_code=MALFORMED_REQUEST)
+                " method.".format(orient=orient)
+            ),
+            error_code=MALFORMED_REQUEST,
+        )
 
 
 def parse_csv_input(csv_input):
@@ -86,7 +138,7 @@ def parse_csv_input(csv_input):
     :param csv_input: A CSV-formatted string representation of a Pandas DataFrame, or a stream
                       containing such a string representation.
     """
-    # pylint: disable=broad-except
+
     try:
         return pd.read_csv(csv_input)
     except Exception:
@@ -94,8 +146,10 @@ def parse_csv_input(csv_input):
             error_message=(
                 "Failed to parse input as a Pandas DataFrame. Ensure that the input is"
                 " a valid CSV-formatted Pandas DataFrame produced using the"
-                " `pandas.DataFrame.to_csv()` method."),
-            error_code=MALFORMED_REQUEST)
+                " `pandas.DataFrame.to_csv()` method."
+            ),
+            error_code=MALFORMED_REQUEST,
+        )
 
 
 def parse_split_oriented_json_input_to_numpy(json_input):
@@ -103,12 +157,14 @@ def parse_split_oriented_json_input_to_numpy(json_input):
     :param json_input: A JSON-formatted string representation of a Pandas DataFrame with split
                        orient, or a stream containing such a string representation.
     """
-    # pylint: disable=broad-except
+
     try:
         json_input_list = json.loads(json_input, object_pairs_hook=OrderedDict)
-        return pd.DataFrame(index=json_input_list['index'],
-                            data=np.array(json_input_list['data'], dtype=object),
-                            columns=json_input_list['columns']).infer_objects()
+        return pd.DataFrame(
+            index=json_input_list["index"],
+            data=np.array(json_input_list["data"], dtype=object),
+            columns=json_input_list["columns"],
+        ).infer_objects()
     except Exception:
         _handle_serving_error(
             error_message=(
@@ -117,15 +173,19 @@ def parse_split_oriented_json_input_to_numpy(json_input):
                 " produced using the `pandas.DataFrame.to_json(..., orient='split')`"
                 " method."
             ),
-            error_code=MALFORMED_REQUEST)
+            error_code=MALFORMED_REQUEST,
+        )
 
 
 def predictions_to_json(raw_predictions, output):
     predictions = _get_jsonable_obj(raw_predictions, pandas_orient="records")
+    wrapper_attr_name = os.environ.get(PREDICTIONS_WRAPPER_ATTR_NAME_ENV_KEY, None)
+    if wrapper_attr_name:
+        predictions = {wrapper_attr_name: predictions}
     json.dump(predictions, output, cls=NumpyEncoder)
 
 
-def _handle_serving_error(error_message, error_code):
+def _handle_serving_error(error_message, error_code, include_traceback=True):
     """
     Logs information about an exception thrown by model inference code that is currently being
     handled and reraises it with the specified error message. The exception stack trace
@@ -134,23 +194,27 @@ def _handle_serving_error(error_message, error_code):
     :param error_message: A message for the reraised exception.
     :param error_code: An appropriate error code for the reraised exception. This should be one of
                        the codes listed in the `mlflow.protos.databricks_pb2` proto.
+    :param include_traceback: Whether to include the current traceback in the returned error.
     """
-    traceback_buf = StringIO()
-    traceback.print_exc(file=traceback_buf)
-    reraise(MlflowException,
-            MlflowException(
-                message=error_message,
-                error_code=error_code,
-                stack_trace=traceback_buf.getvalue()))
+    if include_traceback:
+        traceback_buf = StringIO()
+        traceback.print_exc(file=traceback_buf)
+        traceback_str = traceback_buf.getvalue()
+        e = MlflowException(message=error_message, error_code=error_code, stack_trace=traceback_str)
+    else:
+        e = MlflowException(message=error_message, error_code=error_code)
+    reraise(MlflowException, e)
 
 
-def init(model):
+def init(model: PyFuncModel):
+
     """
     Initialize the server. Loads pyfunc model from the path.
     """
     app = flask.Flask(__name__)
+    input_schema = model.metadata.get_input_schema()
 
-    @app.route('/ping', methods=['GET'])
+    @app.route("/ping", methods=["GET"])
     def ping():  # pylint: disable=unused-variable
         """
         Determine if the container is working and healthy.
@@ -158,9 +222,9 @@ def init(model):
         """
         health = model is not None
         status = 200 if health else 404
-        return flask.Response(response='\n', status=status, mimetype='application/json')
+        return flask.Response(response="\n", status=status, mimetype="application/json")
 
-    @app.route('/invocations', methods=['POST'])
+    @app.route("/invocations", methods=["POST"])
     @catch_mlflow_exception
     def transformation():  # pylint: disable=unused-variable
         """
@@ -168,42 +232,91 @@ def init(model):
         we take data as CSV or json, convert it to a Pandas DataFrame or Numpy,
         generate predictions and convert them back to json.
         """
+
+        # Content-Type can include other attributes like CHARSET
+        # Content-type RFC: https://datatracker.ietf.org/doc/html/rfc2045#section-5.1
+        # TODO: Suport ";" in quoted parameter values
+        type_parts = flask.request.content_type.split(";")
+        type_parts = list(map(str.strip, type_parts))
+        mime_type = type_parts[0]
+        parameter_value_pairs = type_parts[1:]
+        parameter_values = {}
+        for parameter_value_pair in parameter_value_pairs:
+            (key, _, value) = parameter_value_pair.partition("=")
+            parameter_values[key] = value
+
+        charset = parameter_values.get("charset", "utf-8").lower()
+        if charset != "utf-8":
+            return flask.Response(
+                response="The scoring server only supports UTF-8",
+                status=415,
+                mimetype="text/plain",
+            )
+
+        content_format = parameter_values.get("format")
+
         # Convert from CSV to pandas
-        if flask.request.content_type == CONTENT_TYPE_CSV:
-            data = flask.request.data.decode('utf-8')
+        if mime_type == CONTENT_TYPE_CSV and not content_format:
+            data = flask.request.data.decode("utf-8")
             csv_input = StringIO(data)
             data = parse_csv_input(csv_input=csv_input)
-        elif flask.request.content_type in [CONTENT_TYPE_JSON, CONTENT_TYPE_JSON_SPLIT_ORIENTED]:
-            data = parse_json_input(json_input=flask.request.data.decode('utf-8'),
-                                    orient="split")
-        elif flask.request.content_type == CONTENT_TYPE_JSON_RECORDS_ORIENTED:
-            data = parse_json_input(json_input=flask.request.data.decode('utf-8'),
-                                    orient="records")
-        elif flask.request.content_type == CONTENT_TYPE_JSON_SPLIT_NUMPY:
-            data = parse_split_oriented_json_input_to_numpy(flask.request.data.decode('utf-8'))
+        elif mime_type == CONTENT_TYPE_JSON and not content_format:
+            json_str = flask.request.data.decode("utf-8")
+            data = infer_and_parse_json_input(json_str, input_schema)
+        elif (
+            mime_type == CONTENT_TYPE_JSON and content_format == CONTENT_TYPE_FORMAT_SPLIT_ORIENTED
+        ):
+            data = parse_json_input(
+                json_input=StringIO(flask.request.data.decode("utf-8")),
+                orient="split",
+                schema=input_schema,
+            )
+        elif (
+            mime_type == CONTENT_TYPE_JSON
+            and content_format == CONTENT_TYPE_FORMAT_RECORDS_ORIENTED
+        ):
+            data = parse_json_input(
+                json_input=StringIO(flask.request.data.decode("utf-8")),
+                orient="records",
+                schema=input_schema,
+            )
+        elif mime_type == CONTENT_TYPE_JSON_SPLIT_NUMPY and not content_format:
+            data = parse_split_oriented_json_input_to_numpy(flask.request.data.decode("utf-8"))
         else:
             return flask.Response(
-                response=("This predictor only supports the following content types,"
-                          " {supported_content_types}. Got '{received_content_type}'.".format(
-                            supported_content_types=CONTENT_TYPES,
-                            received_content_type=flask.request.content_type)),
+                response=(
+                    "This predictor only supports the following content types and formats:"
+                    " Types: {supported_content_types}; Formats: {formats}."
+                    " Got '{received_content_type}'.".format(
+                        supported_content_types=CONTENT_TYPES,
+                        formats=FORMATS,
+                        received_content_type=flask.request.content_type,
+                    )
+                ),
                 status=415,
-                mimetype='text/plain')
+                mimetype="text/plain",
+            )
 
         # Do the prediction
-        # pylint: disable=broad-except
+
         try:
             raw_predictions = model.predict(data)
+        except MlflowException as e:
+            _handle_serving_error(
+                error_message=e.message, error_code=BAD_REQUEST, include_traceback=False
+            )
         except Exception:
             _handle_serving_error(
                 error_message=(
                     "Encountered an unexpected error while evaluating the model. Verify"
                     " that the serialized input Dataframe is compatible with the model for"
-                    " inference."),
-                error_code=BAD_REQUEST)
+                    " inference."
+                ),
+                error_code=BAD_REQUEST,
+            )
         result = StringIO()
         predictions_to_json(raw_predictions, result)
-        return flask.Response(response=result.getvalue(), status=200, mimetype='application/json')
+        return flask.Response(response=result.getvalue(), status=200, mimetype="application/json")
 
     return app
 
@@ -230,35 +343,3 @@ def _predict(model_uri, input_path, output_path, content_type, json_format):
 def _serve(model_uri, port, host):
     pyfunc_model = load_model(model_uri)
     init(pyfunc_model).run(port=port, host=host)
-
-
-class NumpyEncoder(JSONEncoder):
-    """ Special json encoder for numpy types.
-    Note that some numpy types doesn't have native python equivalence,
-    hence json.dumps will raise TypeError.
-    In this case, you'll need to convert your numpy types into its closest python equivalence.
-    """
-
-    def default(self, o):  # pylint: disable=E0202
-        if isinstance(o, np.generic):
-            return np.asscalar(o)
-        return JSONEncoder.default(self, o)
-
-
-def _get_jsonable_obj(data, pandas_orient="records"):
-    """Attempt to make the data json-able via standard library.
-    Look for some commonly used types that are not jsonable and convert them into json-able ones.
-    Unknown data types are returned as is.
-
-    :param data: data to be converted, works with pandas and numpy, rest will be returned as is.
-    :param pandas_orient: If `data` is a Pandas DataFrame, it will be converted to a JSON
-                          dictionary using this Pandas serialization orientation.
-    """
-    if isinstance(data, np.ndarray):
-        return data.tolist()
-    if isinstance(data, pd.DataFrame):
-        return data.to_dict(orient=pandas_orient)
-    if isinstance(data, pd.Series):
-        return pd.DataFrame(data).to_dict(orient=pandas_orient)
-    else:  # by default just return whatever this is and hope for the best
-        return data
