@@ -19,7 +19,18 @@ from mlflow.tracking.artifact_utils import _download_artifact_from_uri
 from mlflow.models.model import MLMODEL_FILE_NAME
 from mlflow.models.signature import ModelSignature
 from mlflow.models.utils import ModelInputExample, _save_example
-from mlflow.utils.environment import _mlflow_conda_env, _log_pip_requirements
+from mlflow.utils.environment import (
+    _mlflow_conda_env,
+    _get_pip_deps,
+    _validate_env_arguments,
+    _process_pip_requirements,
+    _process_conda_env,
+    _CONSTRAINTS_FILE_NAME,
+    _CONDA_ENV_FILE_NAME,
+    _REQUIREMENTS_FILE_NAME,
+)
+from mlflow.utils.file_utils import write_to
+from mlflow.utils.docstring_utils import format_docstring, LOG_MODEL_PARAM_DOCS
 from mlflow.utils.model_utils import _get_flavor_configuration
 from mlflow.protos.databricks_pb2 import RESOURCE_ALREADY_EXISTS
 from mlflow.tracking._model_registry import DEFAULT_AWAIT_MAX_SLEEP_SECONDS
@@ -72,17 +83,24 @@ def get_underlying_model_flavor(model):
     return _UNKNOWN_MODEL_FLAVOR
 
 
+def get_default_pip_requirements():
+    """
+    :return: A list of default pip requirements for MLflow Models produced by this flavor.
+             Calls to :func:`save_explainer()` and :func:`log_explainer()` produce a pip environment
+             that, at minimum, contains these requirements.
+    """
+    import shap
+
+    return ["shap=={}".format(shap.__version__)]
+
+
 def get_default_conda_env():
     """
     :return: The default Conda environment for
              MLflow Models produced by calls to
              :func:`save_explainer()` and :func:`log_explainer()`.
     """
-    import shap
-
-    pip_deps = ["shap=={}".format(shap.__version__)]
-
-    return _mlflow_conda_env(additional_pip_deps=pip_deps)
+    return _mlflow_conda_env(additional_pip_deps=get_default_pip_requirements())
 
 
 def _load_pyfunc(path):
@@ -119,6 +137,12 @@ def _log_matplotlib_figure(fig, out_file, artifact_path=None):
         fig.savefig(tmp_path)
 
 
+def _get_conda_env_for_underlying_model(underlying_model_path):
+    underlying_model_conda_path = os.path.join(underlying_model_path, "conda.yaml")
+    with open(underlying_model_conda_path, "r") as underlying_model_conda_file:
+        return yaml.safe_load(underlying_model_conda_file)
+
+
 @experimental
 def log_explanation(predict_function, features, artifact_path=None):
     r"""
@@ -259,6 +283,7 @@ def log_explanation(predict_function, features, artifact_path=None):
 
 
 @experimental
+@format_docstring(LOG_MODEL_PARAM_DOCS.format(package_name=FLAVOR_NAME))
 def log_explainer(
     explainer,
     artifact_path,
@@ -268,6 +293,8 @@ def log_explainer(
     signature: ModelSignature = None,
     input_example: ModelInputExample = None,
     await_registration_for=DEFAULT_AWAIT_MAX_SLEEP_SECONDS,
+    pip_requirements=None,
+    extra_pip_requirements=None,
 ):
     """
     Log an SHAP explainer as an MLflow artifact for the current run.
@@ -280,27 +307,12 @@ def log_explainer(
                                         Currently MLflow serialization is only supported for
                                         models of 'sklearn' or 'pytorch' flavors.
 
-    :param conda_env: Either a dictionary representation of a Conda environment or the path to a
-                      Conda environment yaml file. If provided, this decsribes the environment
-                      this model should be run in. At minimum, it should specify the dependencies
-                      contained in :func:`get_default_conda_env()`. If `None`, the default
-                      :func:`get_default_conda_env()` environment is added to the model.
-                      The following is an *example* dictionary representation of a Conda
-                      environment::
-
-                        {
-                            'name': 'mlflow-env',
-                            'channels': ['defaults'],
-                            'dependencies': [
-                                'python=3.6.0',
-                                'shap=0.37.0'
-                            ]
-                        }
-    :param registered_model_name: (Experimental) If given, create a model version under
+    :param conda_env: {{ conda_env }}
+    :param registered_model_name: If given, create a model version under
                                   ``registered_model_name``, also creating a registered model if one
                                   with the given name does not exist.
 
-    :param signature: (Experimental) :py:class:`ModelSignature <mlflow.models.ModelSignature>`
+    :param signature: :py:class:`ModelSignature <mlflow.models.ModelSignature>`
                       describes model input and output :py:class:`Schema <mlflow.types.Schema>`.
                       The model signature can be :py:func:`inferred <mlflow.models.infer_signature>`
                       from datasets with valid model input (e.g. the training dataset with target
@@ -313,7 +325,7 @@ def log_explainer(
                         train = df.drop_column("target_label")
                         predictions = ... # compute model predictions
                         signature = infer_signature(train, predictions)
-    :param input_example: (Experimental) Input example provides one or several instances of valid
+    :param input_example: Input example provides one or several instances of valid
                           model input. The example can be used as a hint of what data to feed the
                           model. The given example will be converted to a Pandas DataFrame and then
                           serialized to json using the Pandas split-oriented format. Bytes are
@@ -321,7 +333,8 @@ def log_explainer(
     :param await_registration_for: Number of seconds to wait for the model version to finish
                             being created and is in ``READY`` status. By default, the function
                             waits for five minutes. Specify 0 or None to skip waiting.
-
+    :param pip_requirements: {{ pip_requirements }}
+    :param extra_pip_requirements: {{ extra_pip_requirements }}
     """
 
     Model.log(
@@ -334,10 +347,13 @@ def log_explainer(
         signature=signature,
         input_example=input_example,
         await_registration_for=await_registration_for,
+        pip_requirements=pip_requirements,
+        extra_pip_requirements=extra_pip_requirements,
     )
 
 
 @experimental
+@format_docstring(LOG_MODEL_PARAM_DOCS.format(package_name=FLAVOR_NAME))
 def save_explainer(
     explainer,
     path,
@@ -346,6 +362,8 @@ def save_explainer(
     mlflow_model=None,
     signature: ModelSignature = None,
     input_example: ModelInputExample = None,
+    pip_requirements=None,
+    extra_pip_requirements=None,
 ):
     """
     Save a SHAP explainer to a path on the local file system. Produces an MLflow Model
@@ -362,25 +380,9 @@ def save_explainer(
                                          Currently MLflow serialization is only supported for
                                          models of 'sklearn' or 'pytorch' flavors.
 
-    :param conda_env: Either a dictionary representation of a Conda environment or the path to a
-                      Conda environment yaml file. If provided, this decsribes the environment
-                      this model should be run in. At minimum, it should specify the dependencies
-                      contained in :func:`get_default_conda_env()`. If `None`, the default
-                      :func:`get_default_conda_env()` environment is added to the model.
-                      The following is an *example* dictionary representation of a Conda
-                      environment::
-
-                        {
-                            'name': 'mlflow-env',
-                            'channels': ['defaults'],
-                            'dependencies': [
-                                'python=3.6.0',
-                                'shap=0.37.0'
-                            ]
-                        }
-
+    :param conda_env: {{ conda_env }}
     :param mlflow_model: :py:mod:`mlflow.models.Model` this flavor is being added to.
-    :param signature: (Experimental) :py:class:`ModelSignature <mlflow.models.ModelSignature>`
+    :param signature: :py:class:`ModelSignature <mlflow.models.ModelSignature>`
                       describes model input and output :py:class:`Schema <mlflow.types.Schema>`.
                       The model signature can be :py:func:`inferred <mlflow.models.infer_signature>`
                       from datasets with valid model input (e.g. the training dataset with target
@@ -393,14 +395,18 @@ def save_explainer(
                         train = df.drop_column("target_label")
                         predictions = ... # compute model predictions
                         signature = infer_signature(train, predictions)
-    :param input_example: (Experimental) Input example provides one or several instances of valid
+    :param input_example: Input example provides one or several instances of valid
                           model input. The example can be used as a hint of what data to feed the
                           model. The given example will be converted to a Pandas DataFrame and then
                           serialized to json using the Pandas split-oriented format. Bytes are
                           base64-encoded.
+    :param pip_requirements: {{ pip_requirements }}
+    :param extra_pip_requirements: {{ extra_pip_requirements }}
     """
     import shap
 
+    _validate_env_arguments(conda_env, pip_requirements, extra_pip_requirements)
+
     if os.path.exists(path):
         raise MlflowException(
             message="Path '{}' already exists".format(path), error_code=RESOURCE_ALREADY_EXISTS,
@@ -444,31 +450,12 @@ def save_explainer(
         else:
             explainer.save(explainer_output_file_handle)
 
-    conda_env_subpath = "conda.yaml"
-    if conda_env is None:
-        conda_env = get_default_conda_env()
-    elif not isinstance(conda_env, dict):
-        with open(conda_env, "r") as f:
-            conda_env = yaml.safe_load(f)
-
-    # merging the conda environment generated by serializing the underlying model
-    if underlying_model_path is not None:
-        underlying_model_conda_path = os.path.join(underlying_model_path, "conda.yaml")
-        with open(underlying_model_conda_path, "r") as underlying_model_conda_file:
-            underlying_model_conda_env = yaml.safe_load(underlying_model_conda_file)
-        conda_env = _merge_environments(conda_env, underlying_model_conda_env)
-
-    with open(os.path.join(path, conda_env_subpath), "w") as f:
-        yaml.safe_dump(conda_env, stream=f, default_flow_style=False)
-
-    _log_pip_requirements(conda_env, path)
-
     pyfunc.add_to_model(
         mlflow_model,
         loader_module="mlflow.shap",
         model_path=explainer_data_subpath,
         underlying_model_flavor=underlying_model_flavor,
-        env=conda_env_subpath,
+        env=_CONDA_ENV_FILE_NAME,
     )
 
     mlflow_model.add_flavor(
@@ -480,6 +467,38 @@ def save_explainer(
 
     mlflow_model.save(os.path.join(path, MLMODEL_FILE_NAME))
 
+    if conda_env is None:
+        if pip_requirements is None:
+            default_reqs = get_default_pip_requirements()
+            # To ensure `_load_pyfunc` can successfully load the model during the dependency
+            # inference, `mlflow_model.save` must be called beforehand to save an MLmodel file.
+            inferred_reqs = mlflow.models.infer_pip_requirements(
+                path, FLAVOR_NAME, fallback=default_reqs,
+            )
+            default_reqs = sorted(set(inferred_reqs).union(default_reqs))
+        else:
+            default_reqs = None
+        conda_env, pip_requirements, pip_constraints = _process_pip_requirements(
+            default_reqs, pip_requirements, extra_pip_requirements,
+        )
+    else:
+        conda_env, pip_requirements, pip_constraints = _process_conda_env(conda_env)
+
+    if underlying_model_path is not None:
+        underlying_model_conda_env = _get_conda_env_for_underlying_model(underlying_model_path)
+        conda_env = _merge_environments(conda_env, underlying_model_conda_env)
+        pip_requirements = _get_pip_deps(conda_env)
+
+    with open(os.path.join(path, _CONDA_ENV_FILE_NAME), "w") as f:
+        yaml.safe_dump(conda_env, stream=f, default_flow_style=False)
+
+    # Save `constraints.txt` if necessary
+    if pip_constraints:
+        write_to(os.path.join(path, _CONSTRAINTS_FILE_NAME), "\n".join(pip_constraints))
+
+    # Save `requirements.txt`
+    write_to(os.path.join(path, _REQUIREMENTS_FILE_NAME), "\n".join(pip_requirements))
+
 
 # Defining save_model (Required by Model.log) to refer to save_explainer
 save_model = save_explainer
@@ -507,6 +526,13 @@ def _get_conda_and_pip_dependencies(conda_env):
     return conda_deps, pip_deps
 
 
+def _union_lists(l1, l2):
+    """
+    Returns the union of two lists as a new list.
+    """
+    return l1 + [x for x in l2 if x not in l1]
+
+
 def _merge_environments(shap_environment, model_environment):
     """
     Merge conda environments of underlying model and shap.
@@ -514,20 +540,18 @@ def _merge_environments(shap_environment, model_environment):
     :param shap_environment: SHAP conda environment.
     :param model_environment: Underlying model conda environment.
     """
-
     # merge the channels from the two environments and remove the default conda
     # channels if present since its added later in `_mlflow_conda_env`
-
-    merged_conda_channels = list(
-        set(shap_environment["channels"] + model_environment["channels"]) - set(["conda-forge"])
+    merged_conda_channels = _union_lists(
+        shap_environment["channels"], model_environment["channels"]
     )
+    merged_conda_channels = [x for x in merged_conda_channels if x != "conda-forge"]
 
     shap_conda_deps, shap_pip_deps = _get_conda_and_pip_dependencies(shap_environment)
     model_conda_deps, model_pip_deps = _get_conda_and_pip_dependencies(model_environment)
 
-    merged_conda_deps = list(set(shap_conda_deps + model_conda_deps))
-    merged_pip_deps = list(set(shap_pip_deps + model_pip_deps))
-
+    merged_conda_deps = _union_lists(shap_conda_deps, model_conda_deps)
+    merged_pip_deps = _union_lists(shap_pip_deps, model_pip_deps)
     return _mlflow_conda_env(
         additional_conda_deps=merged_conda_deps,
         additional_pip_deps=merged_pip_deps,
